require 'active_support/core_ext/object/try'
require 'faraday'
require 'faraday_middleware'
require 'hashie'
require 'atlantis/version'
require 'atlantis/request_error'

module Atlantis
  class AppClient

    attr_reader :url, :http

    def initialize
      @url = ENV['ATLANTIS_URL'] || 'http://atlantis.sodalis.com.au'
      @http = Faraday.new(url: @url, ssl: {verify: false}) do |builder|
        builder.response :mashify
        builder.response :json, :content_type => /\bjson$/
        builder.request :json
        builder.request :basic_auth, ENV['ATLANTIS_API_KEY'], ENV['ATLANTIS_API_SECRET']
        builder.options[:read_timeout] = 4
        builder.options[:open_timeout] = 2
        builder.adapter :excon
      end

      @http.headers = {
        accept:  'application/json',
        user_agent: "Atlantis Ruby Gem #{Atlantis::VERSION}",
        "Content-Type" => "application/json"
      }
    end

    def prepare file, options
    	_post '/upload/prepare', options
    end

    def create attributes
      _post "/documents", attributes
    end

    def _delete path, params={}
     request :delete, path, params
    end

    def _get path, params={}
     request :get, path, params
    end

    def _post path, params={}
     request :post, path, params
    end

    def _put path, params={}
     request :put, path, params
    end

    def request method, path, params
      response = http.send(method, path, params)

      case response.status
      when 200..299
        response.body
      when 404, 410
        raise RequestError::NotFound.new(response)
      when 401
        raise RequestError::Unauthorized.new(response)
      else
        raise RequestError.new(response)
      end
    end
  end
end
